<?php

namespace App\Jobs;

use App\Models\Access\User\User;
use App\Notifications\Backend\Alert\AlertCreated;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use LaravelFCM\Facades\FCM;
use LaravelFCM\Message\OptionsBuilder;
use LaravelFCM\Message\PayloadDataBuilder;
use LaravelFCM\Message\PayloadNotificationBuilder;
use LaravelFCM\Response\Exceptions\InvalidRequestException;
use Nexmo\Client\Exception\Exception;
use Nexmo\Laravel\Facade\Nexmo;

class SendAlertCreated implements ShouldQueue {
	use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

	/**
	 * Create a new job instance.
	 *
	 * @return void
	 */

	protected $alert;

	public function __construct($alert) {
		$this->alert = $alert;
	}

	/**
	 * Execute the job.
	 *
	 * @return void
	 */
	public function handle() {
		if ($this->attempts() > 1) {
			return;
		}
		$users = User::where('organization', $this->alert->organization)
			->where('approve', 1)
			->where('status', 1)
			->get();

		if ($users->count() > 0) {
			foreach ($users as $user) {
				usleep(5000);
				if ($this->alert->push == 1) {
					if ($user->fcmtoken != "") {
						$optionBuilder = new OptionsBuilder();
						$optionBuilder->setTimeToLive(60 * 20);
						$option = $optionBuilder->build();
						$notificationBuilder = new PayloadNotificationBuilder('');

						$notificationBuilder->setBody($this->alert->content)
							->setSound('default');
						$notificationBuilder->setTitle($this->alert->title);
						$notificationBuilder->setClickAction("AlertCreated");
						$notification = $notificationBuilder->build();

						$dataBuilder = new PayloadDataBuilder();
						$dataBuilder->addData(['alertInfo' => $this->alert]);
						$data = $dataBuilder->build();

						try {
							$downstreamResponse = FCM::sendTo($user->fcmtoken, $option, $notification, $data);
						} catch (InvalidRequestException $exception) {
							\Log::info('Error sending fcm message: ' . $exception);
						}
					}
				}

				if ($this->alert->text == 1) {
					$message = $this->alert->title . "\r\n" . $this->alert->content;

					if ($user->phonenumber != null) {
						$phonenumber = "";
						if (strlen($user->phonenumber) > 0) {
							if (strlen($user->phonenumber) <= 10) {
								$phonenumber = "1" . $user->phonenumber;
							} else {
								$phonenumber = $user->phonenumber;
							}
							try {
//'from' => '12018126004',
								\Log::info('Sending to : ' . $phonenumber);
								Nexmo::message()->send([
									'to' => $phonenumber,
									'from' => env("PHONE_NUMBER", "12018126004"),
									'text' => $message,
								]);
							} catch (Exception $exception) {
								\Log::info('Error sending phone message: ' . $exception);
							}
						}
					}
				}

				if ($this->alert->email == 1) {
					\Log::info('Sending email to : ' . $user->email);
					$user->notify(new AlertCreated($this->alert));
				}
			}
		}
	}
}
